/* MoreBalance - load balancer and networking tool
 * Copyright (C) 2002 - 2006 Josef Spillner <josef@coolprojects.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* MoreBalance includes */
#include "filter.h"
#include "spec.h"

/* System includes */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <regex.h>
#ifdef USE_PCRE
#include <pcre.h>
#endif

struct filter_t
{
	char *pattern;
	char *service;
};
typedef struct filter_t filter;

static filter *filterlist = NULL;
static int filters = 0;

void filter_setup(const char *pattern, const char *service)
{
	filterlist = (filter*)realloc(filterlist, (filters + 1) * sizeof(filter));
	filterlist[filters].pattern = strdup(pattern);
	filterlist[filters].service = strdup(service);
	filters++;
}

void filter_setupall(void)
{
	filter_setup("GET /[[:alnum:]]*/ HTTP/1", "http");
	/*filter_setup("MAIL FROM:", "smtp");*/ /* client-to-server! */
	/*filter_setup("<LOGIN TYPE=\'[[:lower:]]*\'>", "ggz");*/ /* client-to-server! */
	filter_setup("\\[0x00\\]\\[0x00\\]\\[0x01\\]\\[0x28\\]\\[0x04\\]\\[0xd2\\]\\[0x16\\]\\[0x2f\\]", "postgres");
}

int filter_tell(const char *extract)
{
	int i, ret;
#ifdef USE_PCRE
	pcre *pex;
	const char *errptr;
	int erroffset;
#else
	regex_t ex;
#endif

	if(!filterlist) filter_setupall();

	for(i = 0; i < filters; i++)
	{
#ifdef USE_PCRE
		pex = pcre_compile(filterlist[i].pattern, 0, &errptr, &erroffset, NULL);
		if((pex) && (!errptr))
		{
			ret = pcre_exec(pex, NULL, extract, strlen(extract), 0, 0, NULL, 0);
			ret = -ret;
		}
		else ret = -1;
		if(pex) pcre_free(pex);
#else
		ret = regcomp(&ex, filterlist[i].pattern, REG_NOSUB | REG_EXTENDED);
		if(!ret)
		{
			ret = regexec(&ex, extract, strlen(extract), NULL, 0);
		}
		else ret = -1;
#endif
		if(!ret) return 1;
		else if(ret == -1) fprintf(stderr, "filter: Regexp for %s contains errors.\n", filterlist[i].service);
	}
	return 0;
}

int filter_tellbinary(unsigned char *array, int length)
{
	char *tmp = NULL;
	int size = 0;
	int i, ret;
	const char hex[] = "0123456789abcdef";

	for(i = 0; i < length; i++)
	{
		if((array[i] < 32) || (array[i] == 127))
		/*if(1)*/
		{
			tmp = (char*)realloc(tmp, size + 7);
			tmp[size + 0] = '[';
			tmp[size + 1] = '0';
			tmp[size + 2] = 'x';
			tmp[size + 3] = hex[array[i] / 16];
			tmp[size + 4] = hex[array[i] % 16];
			tmp[size + 5] = ']';
			tmp[size + 6] = 0;
			size += 6;
		}
		else
		{
			tmp = (char*)realloc(tmp, size + 2);
			tmp[size + 0] = array[i];
			tmp[size + 1] = 0;
			size += 1;
		}
	}

	ret = filter_tell(tmp);

	if(tmp) free(tmp);

	return ret;
}


